package compiler

import (
	"git.urbach.dev/cli/q/src/asm"
	"git.urbach.dev/cli/q/src/asmc"
	"git.urbach.dev/cli/q/src/core"
)

// finalize generates the final machine code.
func (r *Result) finalize() {
	// This will be the entry point of the executable.
	// The only job of the entry function is to call `main` and exit cleanly.
	// The reason we call `main` instead of using `main` itself is to place
	// a return address on the stack, which allows return statements in `main`.
	final := asm.Assembler{
		Instructions: make([]asm.Instruction, 0, r.InstructionCount+8),
		Data:         make(map[string][]byte, r.DataCount),
		Param: asm.Param{
			Label:            make([]asm.Label, 0, r.LabelCount),
			Number:           make([]asm.Number, 0, r.NumberCount),
			Register:         make([]asm.Register, 0, r.RegisterCount),
			RegisterLabel:    make([]asm.RegisterLabel, 0, r.RegisterLabelCount),
			RegisterNumber:   make([]asm.RegisterNumber, 0, r.RegisterNumberCount),
			RegisterRegister: make([]asm.RegisterRegister, 0, r.RegisterRegisterCount),
			Memory:           make([]asm.Memory, 0, r.MemoryCount),
			MemoryLabel:      make([]asm.MemoryLabel, 0, r.MemoryLabelCount),
			MemoryNumber:     make([]asm.MemoryNumber, 0, r.MemoryNumberCount),
			MemoryRegister:   make([]asm.MemoryRegister, 0, r.MemoryRegisterCount),
		},
	}

	r.Traversed = make(map[*core.Function]bool, len(r.Functions))

	// This will place the init function immediately after the entry point
	// and also add everything the init function calls recursively.
	r.eachFunction(r.Init, r.Traversed, func(f *core.Function) {
		final.Merge(&f.Assembler)

		for _, library := range f.DLLs {
			for _, fn := range library.Functions {
				r.DLLs = r.DLLs.Append(library.Name, fn)
			}
		}
	})

	final.Merge(&r.Functions["core.crash"].Assembler)
	r.Code, r.Data = asmc.Finalize(&final, r.DLLs)
}
