package core

import (
	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/token"
)

// ExecuteToken performs an operation on a register with the given leaf operand.
func (f *Function) ExecuteToken(operation token.Token, register cpu.Register, operand token.Token) error {
	switch operand.Kind {
	case token.Identifier:
		name := operand.Text(f.File.Bytes)
		variable := f.VariableByName(name)

		if variable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, operand.Position)
		}

		defer f.UseVariable(variable)
		return f.ExecuteRegisterRegister(operation, register, variable.Value.Register)

	case token.Number, token.Rune:
		number, err := f.ToNumber(operand)

		if err != nil {
			return err
		}

		return f.ExecuteRegisterNumber(operation, register, number)

	case token.String:
		if operation.Kind == token.Assign {
			value, err := f.EvaluateToken(operand)

			if err != nil {
				return err
			}

			f.ValueToRegister(value, register)
			return err
		}
	}

	return errors.New(errors.NotImplemented, f.File, operation.Position)
}
