package scanner

import (
	"git.urbach.dev/cli/q/src/core"
	"git.urbach.dev/cli/q/src/errors"
	"git.urbach.dev/cli/q/src/fs"
	"git.urbach.dev/cli/q/src/token"
)

// scanConst scans a block of constants.
func (s *Scanner) scanConst(file *fs.File, tokens token.List, i int) (int, error) {
	i++

	if tokens[i].Kind != token.BlockStart {
		return i, errors.New(errors.MissingBlockStart, file, tokens[i].Position)
	}

	i++
	prefix := file.Package + "."
	var stack []string

	for i < len(tokens) {
		switch tokens[i].Kind {
		case token.Identifier:
			name := tokens[i].Text(file.Bytes)
			i++

			switch tokens[i].Kind {
			case token.Number:
				s.constants <- &core.Constant{
					Name:  prefix + name,
					Token: tokens[i],
					File:  file,
				}

			case token.BlockStart:
				stack = append(stack, prefix)
				prefix += name + "."

			default:
				return i, errors.New(errors.NotImplemented, file, tokens[i].Position)
			}

		case token.BlockEnd:
			if len(stack) > 0 {
				prefix = stack[len(stack)-1]
				stack = stack[:len(stack)-1]
				i++
				continue
			}

			return i, nil
		}

		i++
	}

	return i, errors.New(errors.MissingBlockEnd, file, tokens[i].Position)
}
